﻿using System;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace Post12085
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);

            InMemoryFeatureLayer layer = new InMemoryFeatureLayer(new FeatureSourceColumn[] { new FeatureSourceColumn("Color", "string", 100) }, new BaseShape[] { });
            Feature f1 = new Feature(0, 0);
            f1.ColumnValues.Add("Color", "Green");
            Feature f2 = new Feature(40, 40);
            f2.ColumnValues.Add("Color", "Black");
            layer.InternalFeatures.Add(f1);
            layer.InternalFeatures.Add(f2);
            layer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(new MyPointStyle() { MapUnit = GeographyUnit.DecimalDegree });
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay o = new LayerOverlay();
            o.Layers.Add(layer);

            winformsMap1.Overlays.Add(o);

            winformsMap1.CurrentExtent = new RectangleShape(-50, 50, 50, -50);


            winformsMap1.Refresh();
        }
    }

    public class MyPointStyle : PointStyle
    {
        private GeographyUnit mapUnit;
        double maximumScale = double.MaxValue;
        double minimumScale = 0;
        double maximumSize = 20;
        double minimumSize = 1;

        public MyPointStyle()
            : base()
        {
        }

        public GeographyUnit MapUnit
        {
            get { return mapUnit; }
            set
            {
                mapUnit = value;
            }
        }

        protected override System.Collections.ObjectModel.Collection<string> GetRequiredColumnNamesCore()
        {
            Collection<string> colors = new Collection<string>();
            colors.Add("Color");
            return colors;
        }


        protected override void DrawCore(System.Collections.Generic.IEnumerable<Feature> features, GeoCanvas canvas, System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInThisLayer, System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInAllLayers)
        {
            //ExtentHelper.GetBoundingBoxOfItems(features);
            //  double currentScale = ExtentHelper.GetScale(ExtentHelper.GetBoundingBoxOfItems(features), canvas.Width, //mapUnit);
            double currentScale = ExtentHelper.GetScale(canvas.CurrentWorldExtent, canvas.Width, mapUnit);
            if (currentScale > maximumScale) { currentScale = maximumScale; }
            if (currentScale < minimumScale) { currentScale = minimumScale; }

            int PointSize = (int)(maximumSize - ((currentScale - minimumScale) * (maximumSize - minimumSize)) /
                                (maximumScale - minimumScale));
            int PointSize2 = (int)(PointSize * 0.9);
            int PointSize3 = (int)(PointSize * 0.6);
            GeoPen gp = new GeoPen();

            Vertex v1 = new Vertex(), v2 = new Vertex(), v3 = new Vertex();
            foreach (Feature feature in features)
            {

                ScreenPointF screenCenterPoint = ExtentHelper.ToScreenCoordinate(canvas.CurrentWorldExtent, feature, canvas.Width, canvas.Height);

                v1 = new Vertex(screenCenterPoint.X, screenCenterPoint.Y);
                v2 = new Vertex(screenCenterPoint.X - 10, screenCenterPoint.Y);
                v3 = new Vertex(screenCenterPoint.X, screenCenterPoint.Y + 10);
                gp = new GeoPen(GeoColor.SimpleColors.Black, (float)0.001);



                PointShape pv1 = ExtentHelper.ToWorldCoordinate(canvas.CurrentWorldExtent, new ScreenPointF((float)v1.X, (float)v1.Y), canvas.Width, canvas.Height);
                PointShape pv2 = ExtentHelper.ToWorldCoordinate(canvas.CurrentWorldExtent, new ScreenPointF((float)v2.X, (float)v2.Y), canvas.Width, canvas.Height);
                PointShape pv3 = ExtentHelper.ToWorldCoordinate(canvas.CurrentWorldExtent, new ScreenPointF((float)v3.X, (float)v3.Y), canvas.Width, canvas.Height);

                RingShape RTriangle = new RingShape();
                RTriangle.Vertices.Add(new Vertex(pv1));
                RTriangle.Vertices.Add(new Vertex(pv2));
                RTriangle.Vertices.Add(new Vertex(pv3));
                RTriangle.Vertices.Add(new Vertex(pv1));


                //string s = "Geocolor.SimpleColor." + feature.ColumnValues["Color"].ToString();

                Color c = Color.FromName(feature.ColumnValues["Color"].ToString().Split('-')[0].ToString());


                GeoSolidBrush gb = new GeoSolidBrush();
                if (feature.ColumnValues["Color"].ToString().Split('-')[0].ToString() == "Green")
                {
                    gb = new GeoSolidBrush(GeoColor.FromArgb(255, c.R, c.G, c.B));
                }
                else
                {
                    gb = new GeoSolidBrush(GeoColor.FromArgb(255, c.R, c.G, c.B));
                }


                canvas.DrawArea(new Feature(new PolygonShape(RTriangle)), gp, gb, DrawingLevel.LevelOne);
            }
        }
    }
}
